// This is the part of MPFRCPP, the C++ interface for MPFR library.

// Copyright (C) 2006 Alexey Beshenov <bav.272304@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301 USA.

#ifndef __CONSTRUCTORS_CPP
#define __CONSTRUCTORS_CPP

//------------------------------------------------------------
// MPFRCPP CONSTRUCTORS
//------------------------------------------------------------

// This file must be included inside the MPFR class namespace!

// Without giving mpc_x value:
MPFR(const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    if (SET_INITD_TO_0 == true) { mpfr_set_ui(mpc_x, 0, frs2mprnd(rnd)); }
/* else { set number to NaN } */
    }


// With assignment:
MPFR(const MPFR &a)
    {
    round_mode(a.round_mode());
    base(a.base());
    mpfr_init2(mpc_x, a.prec());
    mpfr_set (mpc_x, a.mpc_x, mpc_round_mode);
    }


MPFR(const mpfr_t &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set(mpc_x, a, mpc_round_mode);
    }


MPFR(const unsigned long int &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_ui(mpc_x, a, mpc_round_mode);
    }


MPFR(const unsigned int &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_ui(mpc_x, a, mpc_round_mode);
    }


MPFR(const long int &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_si(mpc_x, a, mpc_round_mode);
    }


MPFR(const int &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_si(mpc_x, a, mpc_round_mode);
    }


MPFR(const double &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_d(mpc_x, a, mpc_round_mode);
    }


MPFR(const long double &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_ld(mpc_x, a, mpc_round_mode);
    }


MPFR(const mpz_t &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_z(mpc_x, a, mpc_round_mode);
    }


MPFR(const mpq_t &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_q(mpc_x, a, mpc_round_mode);
    }


MPFR(const mpf_t &a, const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, mpf_get_prec(a));
    mpfr_set_f(mpc_x, a, mpc_round_mode);
    }


#ifdef GMP_CPP_INTERFACE
MPFR(const mpz_class &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_z(mpc_x, a.get_mpz_t(), mpc_round_mode);
    }


MPFR(const mpq_class &a, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, prec);
    mpfr_set_q(mpc_x, a.get_mpq_t(), mpc_round_mode);
    }


MPFR(const mpf_class &a, const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    base(DEFAULT_BASE);
    mpfr_init2(mpc_x, a.get_prec());
    mpfr_set_f(mpc_x, a.get_mpf_t(), mpc_round_mode);
    }
#endif

MPFR(const std::string &a, const int b, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    round_mode(rnd);
    mpfr_init2(mpc_x, prec);
    if (b == 0 || (b >= 2 && b <= 36)) {
        int c = mpfr_set_str(mpc_x, a.c_str(), b, mpc_round_mode);
        if (c == -1) { throw error(__FILE__, __LINE__, error::invalid_num_str); }
        else { base(b); }
        }
    else { throw error(__FILE__, __LINE__, error::invalid_base); }
    }


// Destructor:

~MPFR() { mpfr_clear(mpc_x); }

#endif                                            // __CONSTRUCTORS_CPP
