// This is the part of MPFRCPP, the C++ interface for MPFR library.

// Copyright (C) 2006 Alexey Beshenov <bav.272304@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301 USA.

#ifndef __ERROR_HPP
#define __ERROR_HPP

//------------------------------------------------------------
// MPFRCPP ERROR HANDLING
//------------------------------------------------------------

// This file must be included inside the MPFR class namespace!

class error
    {

    public:

        enum error_type
            {
            unknown = -1,
            invalid_arg = 0,
            invalid_num_str = 1,
            invalid_base = 2,
            cmp_nan = 3,
            conv_nf = 4
            };

        error (const std::string &file, const unsigned int line, const error::error_type error=error::unknown) {

            std::cerr << "ERROR @ " << file << ":" << line << " : ";

            std::cerr << "``";

            switch (error) {

                case error::invalid_arg:
                    std::cerr << "invalid argument";
                    break;

                case error::invalid_num_str:
                    std::cerr << "invalid number string";
                    break;

                case error::invalid_base:
                    std::cerr << "invalid number base";
                    break;

                case error::cmp_nan:
                    std::cerr << "comparison with NaN operand(s)";
                    break;

                case error::conv_nf:
                    std::cerr << "number does not fits() in conversion target type";
                    break;

                default: std::cerr << "unknown";

                }

            std::cerr << "''";

            std::cerr << std::endl;

            }

    };

#endif
